#!/usr/bin/env bash
#
# Description: 気象庁JSONから天気を表示する
# Create: 2022-04-12
# Update: 2022-07-26 Version.2

LANG=ja_JP.UTF-8
LC_TIME=ja_JP.UTF-8
timeout=3

fc3f() { # lemma1': 指定した地域(_area)中の区分(_area3f)のweatherCodes(最大3日)を取得する
    [ -z "$1" ] && return
    local i wdt dt wi wf

    wdt=( $(jq '.[0].timeSeries[0].timeDefines[]' <<<$1 | tr -d \" ) ) # 予報日時
    wi=( $(jq ".[0].timeSeries[0].areas[] | select(.area.name == \"${_area3f}\") .weathers[]" <<<$1 | tr -d \") ) # 天気予報

    i=0
    for dt in ${wdt[@]}; do
        wf=${wi[$i]//　/、}
        wf=${wf//、から/から}
        wf=${wf//、まで/まで}
        printf "%s\t%s\n" $dt "${wf}"
        let i+=1
    done
}

fc3t() { # lemma3': 指定した観測値(_area)の区分(_area3t)の最低/最高気温(最大3日)を取得する
    [ -z "$1" ] && return
    local i wdt dt bdt wi wt

    wdt=( $(jq '.[0].timeSeries[2].timeDefines[]' <<<$1 | tr -d \" ) )
    wi=( $(jq ".[0].timeSeries[2].areas[] | select(.area.name == \"${_area3t}\") .temps[]" <<<$1) )

    i=0
    for dt in ${wdt[@]} ; do # 1日(今日,明日分だけ)の中に2回気温予測がある
        if [ -z "$bdt" ]; then
            wt=${wi[$i]}
            bdt=$dt
        else # 連続する時系列で、${dt%T*} = ${bdt%T}であることが前提の処理
            [ "$wt" = "${wi[$i]}" ] && wt='""'  # 同じ値だったら、最低気温は空にする
            printf "%s\t%s\t%s\n"  ${bdt} $wt ${wi[$i]}
            bdt=""
        fi
        let i+=1
    done
}

get_forecast() { # get_forecast
    local forecast="-"
    forecast=$(curl -f -m ${timeout} -s https://www.jma.go.jp/bosai/forecast/data/forecast/${_area_code}.json)
    if ! jq . <<<$_forecast > /dev/null 2>&1 ; then
        printf ""
    else
        printf "%s\n" "$forecast"
    fi
}

forecast3() { # for Debugging
    local forecast=$(get_forecast)
    if [ -z "$forecast" ]; then
        echo "天気を取得できませんでした。"
        return
    fi
    [ $DEBUG -gt 0 ] && echo "[DEBUG] 3日間 ${_area}:${_area3f} の予報"
    fc3f "$forecast"; echo
    [ $DEBUG -gt 0 ] && echo "[DEBUG] 3日間 ${_area}:${_area3t} の気温"
    fc3t "$forecast"; echo
}

getweather1() { # $day $forecast
    local d=$1
    local forecast="$2"
    local f=$(awk "\$1 ~ /^$d/ { gsub(\"\\\"\", \"\", \$2); gsub(\"\\\"\", \"\", \$3);
        if (\$2) printf(\"%s\\n\", \$2); }" <(fc3f "$forecast") )
    local t=$(awk "\$1 ~ /^$d/ { gsub(\"\\\"\", \"\", \$2); gsub(\"\\\"\", \"\", \$3);
        if (\$2 || \$3) printf(\"%s %s\\n\", \$2, \$3); }" <(fc3t "$forecast") )
    printf "%s\t%s\n" "${f}" "${t}"
}

talk_weather() { # weather.bashのstand alone機能
    local d wf=${1-3}
    local dm=("今日" "明日" "明後日")
    local _days=( $($DATE --iso               )
                  $($DATE --iso --date='1 day')
                  $($DATE --iso --date='2 day') )
    local tm=$( ${DATE} +"%_m月%_d日、%a曜日、%_H時%_M分" | sed 's/ //g')
    local forecast=$(get_forecast)

    if [ -z "$forecast" ]; then
        echo "${tm}、天気を取得できませんでした。"
    else
        echo "${tm}、${_area}の天気予報をお伝えします。"
        for (( i=0; i<wf; i++ )) ; do
            set - $(getweather1 ${_days[$i]} "$forecast")
            case $# in
                1) printf "%sは、%s です。\n" "${dm[$i]}" "$1" ;;
                2) printf "%sは、%s。気温は、%d度です。\n" "${dm[$i]}" "$1" "$2" ;;
                3) printf "%sは、%s。最低気温は、%d度。最高気温は、%d度です。\n" "${dm[$i]}" "$1" "$2" "$3" ;;
                *) echo "なんか異常です。" ;;
            esac
        done
    fi
}

tts() {
    local IFS=
    local s=$(xargs -0)
    if [ -x $dry/usr/bin/mplayer ]; then # Google text to speech mplayer
        mplayer -ao alsa -really-quiet -noconsolecontrols "http://translate.google.com/translate_tts?ie=UTF-8&client=tw-ob&tl=ja&q=${s}" > /dev/null 2>&1
    elif [ -x $dry/usr/bin/say ] ; then # say
        echo "$s" | /usr/bin/say -v "Kyoko"
    else
        echo "$s"
    fi
}

if [ -n "$(which gdate)" ]; then
    DATE=$(which gdate) # macではgdate(brew install coreutils)があればgdateを用いるように
else
    DATE=$(which date)
fi

DEBUG=${DEBUG-0}
if [ ! -f "$(dirname $0)/jma_areaconf.bash" ]; then
    echo "No $(dirname $0)/jma_areaconf.bash"
    exit
fi
source "$(dirname $0)/jma_areaconf.bash"

unset ttsq
while getopts :dft OPT ; do
  case $OPT in
    d) let DEBUG+=1 ; echo "DEBUG=$DEBUG" ;;
    f) echo "[DEBUG] forecast3" ; forecast3 ; exit ;; # for Debugging
    t) ttsq=1 ;; # Text to Speech
    *) echo "$0 [-dft] [#]"
       exit ;;
  esac
done
shift $((OPTIND - 1))

if [ -n "$ttsq" -a $DEBUG -eq 0 ]; then # DEBUG>0では喋らない
    talk_weather ${1-2} | tts
else
    talk_weather $1
fi

